@@ -109,7 +109,7 @@ typedef struct {
  */
 
 typedef enum {
-	POOL_PARSE,
+	POOL_PARSE = 0,
 	POOL_BIND,
 	POOL_EXECUTE,
 	POOL_DESCRIBE,
@@ -237,9 +237,10 @@ typedef struct {
 	int major;
 	/* Protocol minor version number */
 	int minor;
-
+#ifdef NOT_USED
 	/* Preferred "master" node id. Only used for SimpleForwardToFrontend. */
 	int preferred_master_node_id;
+#endif
 } POOL_SESSION_CONTEXT;
 
 extern void pool_init_session_context(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend);
@@ -299,13 +300,18 @@ extern void pool_pending_message_reset_previous_message(void);
 extern void pool_pending_message_set_previous_message(POOL_PENDING_MESSAGE *message);
 extern POOL_PENDING_MESSAGE *pool_pending_message_get_previous_message(void);
 extern bool pool_pending_message_exists(void);
+extern const char *pool_pending_message_type_to_string(POOL_MESSAGE_TYPE type);
+extern void pool_check_pending_message_and_reply(POOL_MESSAGE_TYPE type, char kind);
+extern POOL_PENDING_MESSAGE *pool_pending_message_find_lastest_by_query_context(POOL_QUERY_CONTEXT *qc);
 extern void dump_pending_message(void);
 extern void pool_set_major_version(int major);
 extern void pool_set_minor_version(int minor);
 extern int pool_get_minor_version(void);
+#ifdef NOT_USED
 extern void pool_set_preferred_master_node_id(int node_id);
 extern int pool_get_preferred_master_node_id(void);
 extern void pool_reset_preferred_master_node_id(void);
+#endif
 
 #ifdef NOT_USED
 extern void pool_add_prep_where(char *name, bool *map);
